/**************************************************************************/
/* File: close_rtl.c - RT-Linux variant of the close systemcall           */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2009 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2002-2009 Pavel Pisa <pisa@cmp.felk.cvut.cz>             */
/* Funded by OCERA and FRESCOR IST projects                               */
/* Based on CAN driver code by Arnaud Westenberg <arnaud@wanadoo.nl>      */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/

#ifdef CAN_WITH_RTL

#include "../include/can.h"
#include "../include/can_sysdep.h"
#include "../include/main.h"

#include <rtl_malloc.h>
#include <rtl_posixio.h>
#include "../include/can_iortl.h"

#define __NO_VERSION__
#include <linux/module.h>

static inline
int can_release_rtl_common(struct canuser_t *canuser, int file_flags)
{
	struct canque_ends_t *qends;
	struct msgobj_t *obj;
	can_spin_irqflags_t iflags;

	obj = canuser->msgobj;
	qends = canuser->qends;

	can_spin_lock_irqsave(&canuser_manipulation_lock, iflags);
	list_del(&canuser->peers);
	can_spin_unlock_irqrestore(&canuser_manipulation_lock, iflags);
	canuser->qends = NULL;
	canqueue_ends_dispose_rtl(qends, file_flags & O_SYNC);

	can_spin_lock_irqsave(&canuser_manipulation_lock, iflags);
	if(atomic_dec_and_test(&obj->obj_used)){
		can_msgobj_clear_fl(obj,OPENED);
	};
	can_spin_unlock_irqrestore(&canuser_manipulation_lock, iflags);

    #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,50))
	MOD_DEC_USE_COUNT;
    #endif

	return 0;
}


int can_release_rtl_posix(struct rtl_file *fptr)
{
	struct canuser_t *canuser =
		(struct canuser_t *)can_get_rtl_file_private_data(fptr);
	int ret;
	
	if(!canuser || (canuser->magic != CAN_USER_MAGIC)){
		CANMSG("can_release_rtl_posix: bad canuser magic\n");
		return -ENODEV;
	}

	ret=can_release_rtl_common(canuser, fptr->f_flags);

	rt_free(canuser);
	
	return ret;
}

#endif /*CAN_WITH_RTL*/

