/**************************************************************************/
/* File: ioctl_queue.c - CAN driver parameters and versions queries       */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2009 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2002-2009 Pavel Pisa <pisa@cmp.felk.cvut.cz>             */
/* Funded by OCERA and FRESCOR IST projects                               */
/* Based on CAN driver code by Arnaud Westenberg <arnaud@wanadoo.nl>      */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/

#include "../include/can.h"
#include "../include/can_sysdep.h"
#include "../include/main.h"
#include "../include/ioctl.h"


/**
 * can_ioctl_query - query for driver version and features
 * @canuser: pointer to the user/client state structure
 * @what:    select which feature or version is queried
 *
 * Return Value: returns value for queried characteristic or -EINVAL
 *	if there is no reply to the query.
 */

int can_ioctl_query(struct canuser_t *canuser, unsigned long what)
{
	switch(what){
		case CAN_DRV_QRY_BRANCH:
			/* returns driver branch value - "LINC" for LinCAN driver */
			return CAN_DRV_BRANCH;

		case CAN_DRV_QRY_VERSION:
			/* returns driver version as (major<<16) | (minor<<8) | patch */
			return CAN_DRV_VER;

		case CAN_DRV_QRY_MSGFORMAT:
			/* format of canmsg_t structure */
			#ifdef CAN_MSG_VERSION_2
			  return 2;
			#else /*CAN_MSG_VERSION_2*/
			  return 1;
			#endif /*CAN_MSG_VERSION_2*/
	}

	return -EINVAL;
}
