/**************************************************************************/
/* File: read_rtl.c - RT-Linux variant of read systemcall                 */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2009 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2002-2009 Pavel Pisa <pisa@cmp.felk.cvut.cz>             */
/* Funded by OCERA and FRESCOR IST projects                               */
/* Based on CAN driver code by Arnaud Westenberg <arnaud@wanadoo.nl>      */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/

#ifdef CAN_WITH_RTL

#include "../include/can.h"
#include "../include/can_sysdep.h"
#include "../include/main.h"

#include <rtl_posixio.h>
#include "../include/can_iortl.h"

/* This is the 'Normal' read handler for normal transmission messages */
static inline 
ssize_t can_std_read_rtl(struct canque_ends_t *qends, int nonblock_fl, 
			char *buffer, size_t length)
{
	int ret;
	struct canmsg_t *msg_buff = (struct canmsg_t *)buffer;
	int bytes_to_copy;
	struct canque_edge_t *qedge;
	struct canque_slot_t *slot;
	
	ret=canque_test_outslot(qends, &qedge, &slot);
	if(ret<0){
		if (nonblock_fl) {
			return -EAGAIN;
		}
		ret=canque_get_outslot_wait_rtl(qends, &qedge, &slot);
		if(ret<0){
			if (ret==-1) {
				DEBUGMSG("Rx interrupted\n");
				return -EINTR;
			}

			return -EIO;
		}
	}
	*(msg_buff++)=slot->msg;
	canque_free_outslot(qends, qedge, slot);
	bytes_to_copy = length-sizeof(struct canmsg_t);
	
	while (bytes_to_copy > 0) {
		ret=canque_test_outslot(qends, &qedge, &slot);
		if(ret<0)
			break;
		*(msg_buff++)=slot->msg;
		canque_free_outslot(qends, qedge, slot);
		bytes_to_copy -= sizeof(struct canmsg_t);
	}

	return length-bytes_to_copy;
}


ssize_t can_read_rtl_posix(struct rtl_file *fptr, char *buffer,
				size_t length, loff_t *ppos)
{
	struct canuser_t *canuser = 
		(struct canuser_t *)can_get_rtl_file_private_data(fptr);
	struct canque_ends_t *qends;
	int      ret;

	if(!canuser || (canuser->magic != CAN_USER_MAGIC)){
		CANMSG("can_read_rtl_posix: bad canuser magic\n");
		return -ENODEV;
	}

	if (length < sizeof(struct canmsg_t)) {
		DEBUGMSG("Trying to read less bytes than a CAN message, \n");
		DEBUGMSG("this will always return zero.\n");
		return 0;
	}

	qends = canuser->qends;

	/*if (((struct canmsg_t *)buffer)->flags & MSG_RTR)
		ret = can_rtr_read(chip, obj, buffer);
	else*/
		ret = can_std_read_rtl(qends, fptr->f_flags & O_NONBLOCK,
			buffer, length);

	return ret;
}

#endif /*CAN_WITH_RTL*/

